@@ -29,11 +29,11 @@ App::Licensecheck - functions for a simple license checker for source files
 
 =head1 VERSION
 
-Version v3.0.24
+Version v3.0.25
 
 =cut
 
-our $VERSION = version->declare("v3.0.24");
+our $VERSION = version->declare("v3.0.25");
 
 =head1 SYNOPSIS
 
@@ -53,6 +53,26 @@ See the script for casual usage.
 
 =cut
 
+# legacy descriptive names different from SPDX shortnames
+my %SPDX = (
+	'Artistic or GPL' => 'Perl',
+	'BSD-2-clause'    => 'BSD (2 clause)',
+	'BSD-3-clause'    => 'BSD (3 clause)',
+	'BSD-4-clause'    => 'BSD (4 clause)',
+	'CC-BY-SA'        => 'CC by-sa',
+	FTL               => 'Freetype',
+	Expat             => 'MIT/X11 (BSD like)',
+	Libpng            => 'libpng',
+	'MS-PL'           => 'Ms-PL',
+	'public-domain'   => 'Public domain',
+	'Python'          => 'PSF',
+	'SGI-B'           => 'SGI Free Software License B',
+	Zlib              => 'zlib/libpng',
+);
+
+my $ver_prefix_re = qr/(?:version |v\.? ?)?/i;
+my $ver_re        = qr/\d(?:\.\d+)*/;
+
 my $default_check_regex = q!
 	/[\w-]+$ # executable scripts or README like file
 	|\.( # search for file suffix
@@ -236,7 +256,8 @@ sub parse_file
 	my $cleaned_content = clean_comments($content);
 
 	return (
-		$self->parse_license( clean_cruft_and_spaces($cleaned_content) ),
+		$self->parse_license( clean_cruft_and_spaces($cleaned_content) )
+			|| "UNKNOWN",
 		copyright( clean_cruft($cleaned_content) ),
 	);
 }
@@ -295,7 +316,7 @@ sub parse_lines
 	}
 
 	$fh->close;
-	return ( $license, $copyrights );
+	return ( $license || "UNKNOWN", $copyrights );
 }
 
 sub clean_comments
@@ -373,39 +394,67 @@ sub parse_license
 		push @ret, $spdx_extra if $spdx_extra;
 		return @ret;
 	};
+	my $gen_license = sub {
+		my ( $id, $v, $later, $id2, $v2, $later2 ) = @_;
+		my @spdx;
+		my $desc = $SPDX{$id} || $id;
+		$v .= '+' if ($later);
+		push @spdx, $v ? "$id-$v" : $id if ($id);
+		my $desc2;
+		$desc2 = $SPDX{$id2} || $id2 if ($id2);
+		$v2 .= '+' if ($later2);
+		push @spdx, $v2 ? "$id2-$v2" : "$id2" if ($id2);
+		my $legacy = join(
+			' ',
+			$desc,
+			( $v     ? "(v$v)"     : () ),
+			( $desc2 ? "or $desc2" : () ),
+			( $v2    ? "(v$v2)"    : () ),
+		);
+		push @spdx_license, join( ' or ', @spdx );
+		$license = join( ' ', ( $SPDX{$legacy} || $legacy ), $license );
+	};
 
 	#<<<  do not let perltidy touch this (keep long regex on one line)
+
+	my $cc_by_sa = qr/Creative Commons Attribution[ -]Share[ -]?Alike|CC[ -]BY[ -]SA/i;
+
+	# version of AGPL/GPL/LGPL
 	given ($licensetext) {
-		when ( /version (\d(?:\.\d+)*)(?: of the License)?,? or(?: \(at your option\))? version (\d+(?:\.\d+)*)/ ) {
+		when ( /version ($ver_re)(?: of the License)?,? or(?: \(at your option\))? version ($ver_re)/ ) {
 			$gplver = " (v$1 or v$2)";
 			@spdx_gplver = ( $1, $2 );
 		}
-		when ( /version (\d+(?:\.\d+)*)[.,]? (?:\(?only\)?.? )?(?:of the GNU (Affero )?(Lesser |Library )?General Public License )?(as )?published by the Free Software Foundation/i ) {
+		when ( /version ($ver_re)[.,]? (?:\(?only\)?.? )?(?:of the GNU (Affero |Lesser |Library )?General Public License )?(as )?published by the Free Software Foundation/i ) {
 			$gplver      = " (v$1)";
 			@spdx_gplver = ($1)
 		}
-		when ( /GNU (?:Affero )?(?:Lesser |Library )?General Public License (?:as )?published by the Free Software Foundation[;,] version (\d+(?:\.\d+)*)[.,]? /i ) {
+		when ( /GNU (?:Affero |Lesser |Library )?General Public License (?:as )?published by the Free Software Foundation[;,] $ver_prefix_re($ver_re)[.,]? /i ) {
 			$gplver      = " (v$1)";
 			@spdx_gplver = ($1);
 		}
-		when ( /GNU (?:Affero )?(?:Lesser |Library )?General Public License\s*(?:[(),GPL]+)\s*version (\d+(?:\.\d+)*)[ \.]/i ) {
+		when ( /GNU (?:Affero |Lesser |Library )?General Public License ?(?:[(),AGPL]+) ?$ver_prefix_re($ver_re)[ \.]/i ) {
 			$gplver      = " (v$1)";
 			@spdx_gplver = ($1);
 		}
-		when ( /either version (\d+(?:\.\d+)*)(?: of the License)?, or (?:\(at your option\) )?any later version/ ) {
+		when ( /either $ver_prefix_re($ver_re)(?: of the License)?, or (?:\(at your option\) )?any later version/ ) {
 			$gplver      = " (v$1 or later)";
 			@spdx_gplver = ( $1 . '+' );
 		}
-		when ( /GPL\sas\spublished\sby\sthe\sFree\sSoftware\sFoundation,\sversion\s(\d+(?:\.\d+)*)/i ) {
+		when ( /GPL as published by the Free Software Foundation, version ($ver_re)/i ) {
 			$gplver      = " (v$1)";
 			@spdx_gplver = ($1)
 		}
 	}
+
+	# address in AGPL/GPL/LGPL
 	given ($licensetext) {
 		when ( /(?:675 Mass Ave|59 Temple Place|51 Franklin Steet|02139|02111-1307)/i ) {
 			$extrainfo = " (with incorrect FSF address)$extrainfo";
 		}
 	}
+
+	# exception for AGPL/GPL/LGPL
 	given ($licensetext) {
 		when ( /permission (?:is (also granted|given))? to link (the code of )?this program with (any edition of )?(Qt|the Qt library)/i ) {
 			$extrainfo  = " (with Qt exception)$extrainfo";
@@ -413,8 +462,9 @@ sub parse_license
 		}
 	}
 
-	# exclude blurb found in boost license text
+	# generated file
 	given ($licensetext) {
+		# exclude blurb found in boost license text
 		when ( /unless such copies or derivative works are solely in the form of machine-executable object code generated by a source language processor/ ) {
 			break;
 		}
@@ -422,25 +472,49 @@ sub parse_license
 			$license = "GENERATED FILE";
 		}
 	}
+
 	given ($licensetext) {
-		when ( /(are made available|(is free software.? )?you can redistribute (it|them) and(?:\/|\s+)or modify (it|them)|is licensed) under the terms of (version [^ ]+ of )?the (GNU (Library |Lesser )General Public License|LGPL)/i ) {
+		when ( /under(?: the)?(?:GNU)? (AGPL|GPL|LGPL)-?($ver_re)( or later| (?:and|or)(?: \(at your option\))?(?: any)? later(?: version)?)?,? (?:and|or)(?: the| a)? $cc_by_sa(?: version)? ($ver_re)(?: License)?( (?:and|or)(?: \(at your option\))? any later|[,.] Later versions are permitted)?/i ) {
+			$gen_license->( 'CC-BY-SA', $3, $4, $1, $2 );
+		}
+	}
+
+	# LGPL
+	given ($licensetext) {
+		when ( /(are made available|(is free software.? )?you can redistribute (it|them) and[ \/]or modify (it|them)|is licensed) under the terms of (version \S+ of )?the (GNU (Library |Lesser )General Public License|LGPL)/i ) {
 			$license = "LGPL$gplver$extrainfo $license";
 			push @spdx_license, $gen_spdx->('LGPL');
 		}
 		# For Perl modules handled by Dist::Zilla
-		when ( /this is free software,? licensed under:? (?:the )?(?:GNU (?:Library |Lesser )General Public License|LGPL),? version ([\d\.]+)/i ) {
-			$license = "LGPL (v$1) $license";
-			push @spdx_license, "LGPL-$1";
+		when ( /this is free software,? licensed under:? (?:the )?(?:GNU (?:Library |Lesser )General Public License|LGPL),? $ver_prefix_re($ver_re)/i ) {
+			$gen_license->( 'LGPL', $1 );
 		}
 	}
+
+	# AGPL
 	given ($licensetext) {
-		when ( /is free software.? you can redistribute (it|them) and(?:\/|\s+)or modify (it|them) under the terms of the (GNU Affero General Public License|AGPL)/i ) {
+		when ( /is free software.? you can redistribute (it|them) and[ \/]or modify (it|them) under the terms of the (GNU Affero General Public License|AGPL)/i ) {
 			$license = "AGPL$gplver$extrainfo $license";
 			push @spdx_license, $gen_spdx->('AGPL');
 		}
+		# exclude GPL-3 license
+		when ( /GNU Affero General Public License into/i ) {
+			$license = "AGPL (v$1) $license";
+			break;
+		}
+		# exclude MPL-2.0 license
+		when ( /means either [^.]+, the GNU Affero General Public License/i ) {
+			$license = "AGPL (v$1) $license";
+			break;
+		}
+		when ( /AFFERO GENERAL PUBLIC LICENSE(?:,? $ver_prefix_re($ver_re)(,? or(?: any)? (?:later|newer))?)?/i ) {
+			$gen_license->( 'AGPL', $1, $2 );
+		}
 	}
+
+	# GPL
 	given ($licensetext) {
-		when ( /(is free software.? )?you (can|may) redistribute (it|them) and(?:\/|\s+)or modify (it|them) under the terms of (?:version [^ ]+ (?:\(?only\)? )?of )?the GNU General Public License/i ) {
+		when ( /(is free software.? )?you (can|may) redistribute (it|them) and[ \/]or modify (it|them) under the terms of (?:version \S+ (?:\(?only\)? )?of )?the GNU General Public License/i ) {
 			$license = "GPL$gplver$extrainfo $license";
 			push @spdx_license, $gen_spdx->('GPL');
 		}
@@ -449,237 +523,395 @@ sub parse_license
 		$license = "GPL$gplver$extrainfo $license";
 		push @spdx_license, $gen_spdx->('GPL');
 	}
+
+	# LGPL/GPL unversioned
 	given ($licensetext) {
-		when ( /(?:is|may be)\s(?:(?:distributed|used).*?terms|being\s+released).*?\b(L?GPL)\b/ ) {
+		when ( /(?:is|may be) (?:(?:distributed|used).*?terms|being released).*?\b(L?GPL)\b/ ) {
 			my $v = $gplver || ' (unversioned/unknown version)';
 			$license = "$1$v $license";
 			push @spdx_license, $gen_spdx->($1);
 		}
 	}
+
+	# CC
+	given ($licensetext) {
+		when ( /$cc_by_sa(?i: version)? ($ver_re) or ($ver_re)/ ) {
+			$license = "CC BY-SA (v$1 or v$2) $license";
+			push @spdx_license, "CC-BY-SA-$1 or CC-BY-SA-$1";
+		}
+		when ( /$cc_by_sa(?: $ver_prefix_re($ver_re)?)(?: License)?( (?:and|or)(?: \(at your option\))? any later(?: version)|[,.] Later versions are permitted)?(?:,? (?:and|or)(?: the)?(?:GNU)? (AGPL|GPL|LGPL)(?:-?($ver_re)( or later)?)?)?/i ) {
+			$gen_license->( 'CC-BY-SA', $1, $2, $3, $4 );
+		}
+		when ( m<https?creativecommons.org/licenses/by-sa/($ver_re)/( \(or any later version\))?> ) {
+			$gen_license->( 'CC-BY-SA', $1, $2 );
+		}
+	}
+	# CC or GPL
+	given ($licensetext) {
+	}
+
+	# LLGPL
 	given ($licensetext) {
 		when ( /the rights to distribute and use this software as governed by the terms of the Lisp Lesser General Public License|\bLLGPL\b/ ) {
-			$license = "LLGPL $license";
-			push @spdx_license, 'LLGPL';
+			$gen_license->('LLGPL');
 		}
 	}
+
+	# QPL
 	given ($licensetext) {
 		when ( /This file is part of the .*Qt GUI Toolkit. This file may be distributed under the terms of the Q Public License as defined/ ) {
-			$license = "QPL (part of Qt) $license";
+			$gen_license->('QPL');
 		}
 		when ( /may (be distributed|redistribute it) under the terms of the Q Public License/ ) {
-			$license = "QPL $license";
-			push @spdx_license, 'QPL';
+			$gen_license->('QPL');
 		}
 	}
+
+	# MIT
 	given ($licensetext) {
 		when ( /opensource\.org\/licenses\/mit-license\.php/ ) {
-			$license = "MIT/X11 (BSD like) $license";
-			push @spdx_license, 'Expat';
+			$gen_license->('Expat');
 		}
 		when ( /Permission is hereby granted, free of charge, to any person obtaining a copy of this software and(\/or)? associated documentation files \(the (Software|Materials)\), to deal in the (Software|Materials)/ ) {
-			$license = "MIT/X11 (BSD like) $license";
-			push @spdx_license, 'Expat';
+			$gen_license->('Expat');
 		}
 		when ( /Permission is hereby granted, without written agreement and without license or royalty fees, to use, copy, modify, and distribute this software and its documentation for any purpose/ ) {
-			$license = "MIT/X11 (BSD like) $license";
-			push @spdx_license, 'Expat';
+			$gen_license->('Expat');
 		}
 	}
+
+	# ISC
 	given ($licensetext) {
 		when ( /Permission to use, copy, modify, and(\/or)? distribute this software for any purpose with or without fee is hereby granted, provided.*copyright notice.*permission notice.*all copies/ ) {
-		$license = "ISC $license";
-			push @spdx_license, 'ISC';
+			$gen_license->('ISC');
 		}
 	}
+
+	# BSD
 	if ( $licensetext =~ /THIS SOFTWARE IS PROVIDED .*AS IS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY/ ) {
 		given ($licensetext) {
 			when ( /All advertising materials mentioning features or use of this software must display the following acknowledge?ment.*This product includes software developed by/i ) {
-				$license = "BSD (4 clause) $license";
-				push @spdx_license, 'BSD-4-clause';
+				$gen_license->('BSD-4-clause');
 			}
 			when ( /(The name(?:\(s\))? .*? may not|Neither the (names? .*?|authors?) nor the names of( (its|their|other|any))? contributors may) be used to endorse or promote products derived from this software/i ) {
-				$license = "BSD (3 clause) $license";
-				push @spdx_license, 'BSD-3-clause';
+				$gen_license->('BSD-3-clause');
 			}
 			when ( /Redistributions in binary form must reproduce the above copyright notice/i ) {
-				$license = "BSD (2 clause) $license";
-				push @spdx_license, 'BSD-2-clause';
+				$gen_license->('BSD-2-clause');
 			}
 			default {
-				$license = "BSD $license";
-				push @spdx_license, 'BSD';
+				$gen_license->('BSD');
 			}
 		}
 	}
 	elsif ( $licensetext =~ /licen[sc]ebsd(?:-(\d)-clause)?/i ) {
 		if ($1) {
-			$license = "BSD ($1 clause) $license";
-			push @spdx_license, "BSD-$1-clause";
+			$gen_license->("BSD-$1-clause");
 		}
 		else {
-			$license = "BSD $license";
-			push @spdx_license, "BSD";
-		}
-	}
-	given ($licensetext) {
-		when ( /Mozilla Public License,? (?:(?:Version|v\.)\s+)?(\d+(?:\.\d+)?)/ ) {
-			$license = "MPL (v$1) $license";
-			push @spdx_license, "MPL-$1";
-		}
-		when ( /Mozilla Public License,? \((?:Version|v\.) (\d+(?:\.\d+)?)\)/ ) {
-			$license = "MPL (v$1) $license";
-			push @spdx_license, "MPL-$1";
+			$gen_license->('BSD');
 		}
 	}
 
-# match when either:
-# - the text *begins* with "The Artistic license v2.0" which is (hopefully) the actual artistic license v2.0 text.
-# - a license grant is found. i.e something like "this is free software, licensed under the artistic license v2.0"
+	# MPL
 	given ($licensetext) {
-		when ( /(?:^\s*|(?:This is free software, licensed|Released|be used|use and modify this (?:module|software)) under (?:the terms of )?)[Tt]he Artistic License ([v\d.]*\d)/ ) {
-			$license = "Artistic (v$1) $license";
-			push @spdx_license, "Artistic-$1";
+		when ( /Mozilla Public License,? $ver_prefix_re($ver_re)/ ) {
+			$gen_license->( 'MPL', $1 );
+		}
+		when ( /Mozilla Public License,? \($ver_prefix_re($ver_re)\)/ ) {
+			$gen_license->( 'MPL', $1 );
+		}
+	}
+
+	# APAFML
+	given ($licensetext) {
+		when ( /AFM files it accompanies may be used|that the AFM files are not distributed/ ) {
+			$license = "Adobe Postscript AFM License $license";
+			push @spdx_license, "APAFML";
+		}
+	}
+
+	# Adobe-Glyph
+	given ($licensetext) {
+		when ( /and to permit others to do the same, provided that the derived work is not represented as being a copy/ ) {
+			$license = "Adobe Glyph List License $license";
+			push @spdx_license, "Adobe-Glyph";
+		}
+	}
+
+	# Adobe-2006
+	given ($licensetext) {
+		when ( /You agree to indemnify, hold harmless and defend/ ) {
+			$license = "Adobe $license";
+			push @spdx_license, "Adobe-2006";
+		}
+	}
+
+	# Aladdin
+	given ($licensetext) {
+		when ( /This License is not the same as any of the GNU Licenses/ ) {
+			$license = "Aladdin Free Public License $license";
+			push @spdx_license, "Aladdin";
+		}
+		when ( /under the terms of the Aladdin Free Public License/ ) {
+			$license = "Aladdin $license";
+			push @spdx_license, "Aladdin";
+		}
+	}
+
+	# GPL or Aladdin
+	given ($licensetext) {
+		when ( /under the GNU License and Aladdin Free Public License/ ) {
+			$license = "GPL or Aladdin $license";
+			push @spdx_license, "GPL or Aladdin";
+		}
+	}
+
+	# Artistic
+	given ($licensetext) {
+		# either *begins* with "The Artistic license v2.0" (hopefully the actual license)
+		# or some license grant,
+		when ( /(?:^ ?|(?:This is free software, licensed|Released|be used|use and modify this (?:module|software)) under (?:the terms of )?)[Tt]he Artistic License v?($ver_re)/ ) {
+			$gen_license->( 'Artistic', $1 );
 		}
 	}
 	given ($licensetext) {
 		when ( /is free software under the Artistic [Ll]icense/ ) {
-			$license = "Artistic $license";
-			push @spdx_license, 'Artistic';
-		}
-	}
-	given ($licensetext) {
-		when ( /This program is free software; you can redistribute it and\/or modify it under the same terms as Perl itself/ ) {
-			$license = "Perl $license";
-			push @spdx_license, 'Artistic', 'GPL';
-		}
-	}
-	given ($licensetext) {
-		when ( /under the Apache License, Version ([^ ]+)/ ) {
-			$license = "Apache (v$1) $license";
-			push @spdx_license, "Apache-$1";
-		}
-	}
-	given ($licensetext) {
-		when ( /(THE BEER-WARE LICENSE)/i ) {
-			$license = "Beerware $license";
-			push @spdx_license, 'Beerware';
-		}
-	}
-	given ($licensetext) {
-		when ( /distributed under the terms of the FreeType project/i )	{
-			$license = "FreeType $license";
-			push @spdx_license, 'FTL';
-		}
-	}
-	given ($licensetext) {
-		when ( /This source file is subject to version ([^ ]+) of the PHP license/ ) {
-			$license = "PHP (v$1) $license";
-			push @spdx_license, "PHP-$1";
-		}
-	}
-	given ($licensetext) {
-		when ( /under the terms of the CeCILL-([^ ]+) / ) {
-			$license = "CeCILL-$1 $license";
-			push @spdx_license, "CECILL-$1";
-		}
-		when ( /under the terms of the CeCILL / ) {
-			$license = "CeCILL $license";
-			push @spdx_license, 'CECILL';
-		}
-	}
-	given ($licensetext) {
-		when ( /under the SGI Free Software License B/ ) {
-			$license = "SGI Free Software License B $license";
-			push @spdx_license, 'SGI-B';
-		}
-	}
-	given ($licensetext) {
-		when ( /is in the public domain/i ) {
-			$license = "Public domain $license";
-			push @spdx_license, 'public-domain';    # not listed by SPDX
-		}
-	}
-	given ($licensetext) {
-		when ( /terms of the Common Development and Distribution License(, Version ([^(]+))? \(the License\)/ ) {
-			$license = "CDDL " . ( $1 ? "(v$2) " : '' ) . $license;
-			push @spdx_license, 'CDDL' . ( $1 ? "-$2" : '' );
-		}
-	}
-	given ($licensetext) {
-		when ( /Microsoft Permissive License \(Ms-PL\)/ ) {
-			$license = "Ms-PL $license";
-			push @spdx_license, 'MS-PL';
-		}
-	}
-	given ($licensetext) {
-		when ( /Licensed under the Academic Free License version ([\d.]+)/ ) {
-			$license = $1 ? "AFL-$1" : "AFL";
-			push @spdx_license, 'AFL' . ( $1 ? "-$1" : '' );
-		}
-	}
-	given ($licensetext) {
-		when ( /This program and the accompanying materials are made available under the terms of the Eclipse Public License v?([\d.]+)/ ) {
-			$license = $1 ? "EPL-$1" : "EPL";
-			push @spdx_license, 'EPL' . ( $1 ? "-$1" : '' );
+			$gen_license->('Artistic');
 		}
 	}
 
-	# quotes were removed by clean_comments function
+	# Artistic or GPL
+	given ($licensetext) {
+		when ( /This (program )?is free software; you can redistribute it and\/or modify it under the same terms as (the )?Perl( ?5)? (programming |language |system )*itself/ ) {
+			$gen_license->('Artistic', 0, 0, 'GPL');
+		}
+	}
+
+	# Apache
+	given ($licensetext) {
+		when ( /Apache(?: Software)? License(?:,? $ver_prefix_re($ver_re)( or(?: any)? (?:later|newer))?)?(?:(?: or)? [^ ,]*?apache[^ ,]*| \([^(),]+\))*,? or(?: the)?(?:(?: GNU)? General Public License(?: \(GPL\))?|GPL)(?: $ver_prefix_re($ver_re)( or(?: any)? (?:later|newer))?)?/i ) {
+			$gen_license->( 'Apache', $1, $2, 'GPL', $3, $4 );
+		}
+		when ( /Apache(?: Software)? License(?:,? $ver_prefix_re($ver_re)( or(?: any)? (?:later|newer))?)?(?:(?: or)? [^ ,]*?apache[^ ,]*| \([^(),]\))*,? or(?: the)? bsd(?:[ -](\d)-clause)?\b/i ) {
+			$gen_license->( 'Apache', $1, $2, "BSD-$3-clause" );
+		}
+		when ( /Apache(?: Software)? License(?:,? $ver_prefix_re($ver_re)( or(?: any)? (?:later|newer))?)?(?:(?: or)? [^ ,]*?apache[^ ,]*| \([^(),]\))*,? or(?: the)? (?:Expat|MIT)\b/i ) {
+			$gen_license->( 'Apache', $1, $2, 'Expat', $3, $4 );
+		}
+		when ( /Apache(?: Software)? License(?:,? $ver_prefix_re($ver_re)(,? or(?: any)? (?:later|newer))?)?/i ) {
+			$gen_license->( 'Apache', $1, $2 );
+		}
+		when ( m<https?www.apache.org/licenses(?:/LICENSE-($ver_re))?>i ) {
+			$gen_license->( 'Apache', $1 );
+		}
+	}
+
+	# Beerware
+	given ($licensetext) {
+		when ( /(THE BEER-WARE LICENSE)/i ) {
+			$gen_license->('Beerware');
+		}
+	}
+
+	# FTL
+	given ($licensetext) {
+		when ( /distributed under the terms of the FreeType project/i )	{
+			$gen_license->('FTL');
+		}
+	}
+
+	# FSFAP
+	given ($licensetext) {
+		when ( /Copying and distribution of this file, with or without modification, are permitted in any medium without royalty provided the copyright notice and this notice are preserved/i ) {
+			$license = "FSF All Permissive License $license";
+			push @spdx_license, 'FSFAP';
+		}
+	}
+
+	# FSFUL
+	given ($licensetext) {
+		when ( /This configure script is free software; the Free Software Foundation gives unlimited permission to copy, distribute and modify it\./i ) {
+			$license = "FSF Unlimited License $license";
+			push @spdx_license, 'FSFUL';
+		}
+		when ( /This (\w+)(?: (?:file|script))? is free software; the Free Software Foundation gives unlimited permission to copy, distribute and modify it\./i ) {
+			$license = "FSF Unlimited License ($1 derivation) $license";
+			push @spdx_license, "FSFUL~$1";
+		}
+	}
+
+	# FSFULLR
+	given ($licensetext) {
+		when ( /This file is free software; the Free Software Foundation gives unlimited permission to copy and\/or distribute it, with or without modifications, as long as this notice is preserved/i ) {
+			$license = "FSF Unlimited License (with License Retention, $1 derivation) $license";
+			push @spdx_license, 'FSFULLR';
+		}
+		when ( /This (\w+)(?: (?:file|script))?  is free software; the Free Software Foundation gives unlimited permission to copy and\/or distribute it, with or without modifications, as long as this notice is preserved/i ) {
+			$license = "FSF Unlimited License (with License Retention, $1 derivation) $license";
+			push @spdx_license, "FSFULLR~$1";
+		}
+	}
+
+	# JSON
+	given ($licensetext) {
+		when ( /The Software shall be used for Good, not Evil/ ) {
+			$license = "JSON License $license";
+			push @spdx_license, "JSON";
+		}
+	}
+
+	# PHP
+	given ($licensetext) {
+		when ( /This source file is subject to version ($ver_re) of the PHP license/ ) {
+			$gen_license->( 'PHP', $1 );
+		}
+	}
+
+	# CECILL
+	given ($licensetext) {
+		when ( /under the terms of the CeCILL-(\S+) / ) {
+			$gen_license->( 'CeCILL', $1 );
+		}
+		when ( /under the terms of the CeCILL / ) {
+			$gen_license->('CeCILL');
+		}
+	}
+
+	# SGI-B
+	given ($licensetext) {
+		when ( /under the SGI Free Software License B/ ) {
+			$gen_license->('SGI-B');
+		}
+	}
+
+	# public-domain
+	given ($licensetext) {
+		when ( /is in the public domain/i ) {
+			$gen_license->('public-domain');
+		}
+	}
+
+	# CDDL
+	given ($licensetext) {
+		when ( /terms of the Common Development and Distribution License(?:,? $ver_prefix_re($ver_re))?/ ) {
+			$license = "CDDL " . ( $1 ? "(v$1) " : '' ) . $license;
+			push @spdx_license, 'CDDL' . ( $1 ? "-$1" : '' );
+		}
+	}
+
+	# MS-PL
+	given ($licensetext) {
+		when ( /Microsoft Permissive License \(Ms-PL\)/ ) {
+			$gen_license->('MS-PL');
+		}
+	}
+
+	# AFL
+	given ($licensetext) {
+		when ( /Licensed under the Academic Free License(?: $ver_prefix_re($ver_re))?/ ) {
+			$gen_license->( 'AFL', $1 );
+		}
+	}
+
+	# EPL
+	given ($licensetext) {
+		when ( /This program and the accompanying materials are made available under the terms of the Eclipse Public License(?:[ ,-]+$ver_prefix_re($ver_re))?/ ) {
+			$gen_license->( 'EPL', $1 );
+		}
+	}
+
+	# BSL
 	given ($licensetext) {
 		when ( /Permission is hereby granted, free of charge, to any person or organization obtaining a copy of the software and accompanying documentation covered by this license \(the Software\)/ ) {
-			$license = "BSL " . ( $1 ? "(v$2) " : '' ) . $license;
-			push @spdx_license, 'BSL' . ( $1 ? "-$2" : '' );
+			$gen_license->('BSL');
 		}
-		when ( /Boost Software License([ ,-]+Version ([^ ]+)?(\.))/i ) {
-			$license = "BSL " . ( $1 ? "(v$2) " : '' ) . $license;
-			push @spdx_license, 'BSL' . ( $1 ? "-$2" : '' );
+		when ( /Boost Software License(?:[ ,-]+ $ver_prefix_re($ver_re))?/i ) {
+			$gen_license->( 'BSL', $1, $2 );
 		}
 	}
+
+	# PostgreSQL
 	given ($licensetext) {
-		when ( /PYTHON SOFTWARE FOUNDATION LICENSE (VERSION ([^ ]+))/i ) {
-			$license = "PSF " . ( $1 ? "(v$2) " : '' ) . $license;
-			push @spdx_license, 'Python' . ( $1 ? "-$2" : '' );
+		when ( /Permission to use, copy, modify, and distribute this software and its documentation for any purpose, without fee, and without a written agreement is hereby granted, provided that the above copyright notice and this paragraph and the following two paragraphs appear in all copies/i ) {
+			$license = "PostgreSQL $license";
+			push @spdx_license, 'PostgreSQL';
 		}
 	}
+
+	# Python
 	given ($licensetext) {
+		when ( /PYTHON SOFTWARE FOUNDATION LICENSE(?:,? $ver_prefix_re($ver_re))?/i ) {
+			$gen_license->( 'Python', $1, $2 );
+		}
+	}
+
+	# SIL
+	given ($licensetext) {
+		when ( /must be distributed using this license/i ) {
+			$license = "SIL (v1.0) $license";
+			push @spdx_license, 'SIL-1.0';
+		}
+		when ( /must be distributed entirely under this license/i ) {
+			$license = "SIL (v1.1) $license";
+			push @spdx_license, 'SIL-1.1';
+		}
+	}
+
+	# Unicode-strict
+	given ($licensetext) {
+		when ( /Unicode, Inc\. hereby grants the right to freely use the information supplied in this file in the creation of products supporting the Unicode Standard, and to make copies of this file in any form for internal or external distribution as long as this notice remains attached/i ) {
+			$license = "Unicode strict $license";
+			push @spdx_license, 'Unicode-strict';
+		}
+	}
+
+	# Unicode-TOU
+	given ($licensetext) {
+		when ( /Any person is hereby authorized, without fee, to view, use, reproduce, and distribute all documents and files solely for informational purposes in the creation of products supporting the Unicode Standard, subject to the Terms and Conditions herein/i ) {
+			$license = "Unicode Terms of Use $license";
+			push @spdx_license, 'Unicode-TOU';
+		}
+	}
+
+	# Zlib/Libpng
+	given ($licensetext) {
+		when ( /acknowledgment .* in the product documentation is required/ ) {
+			$license = "zlib/libpng License with Acknowledgement $license";
+			push @spdx_license, 'zlib-acknowledgement';
+		}
 		when ( /The origin of this software must not be misrepresented.*Altered source versions must be plainly marked as such.*This notice may not be removed or altered from any source distribution/ ) {
-			$license = "zlib/libpng $license";
-			push @spdx_license, 'Zlib';
+			$gen_license->('Zlib');
 		}
 		when ( /see copyright notice in zlib\.h/ ) {
-			$license = "zlib/libpng $license";
-			push @spdx_license, 'Zlib';
+			$gen_license->('Zlib');
 		}
 		when ( /This code is released under the libpng license/ ) {
-			$license = "libpng $license";
-			push @spdx_license, 'Libpng';
+			$gen_license->('Libpng');
 		}
 	}
+
+	# WTFPL
 	given ($licensetext) {
-		when ( /Do What The Fuck You Want To Public License, Version ([^, ]+)/i ) {
-			$license = "WTFPL (v$1) $license";
-			push @spdx_license, "WTFPL-$1";
+		when ( /Do What The Fuck You Want To Public License,? $ver_prefix_re($ver_re)/i ) {
+			$gen_license->( 'WTFPL', $1 );
 		}
 	}
 	given ($licensetext) {
 		when ( /Do what The Fuck You Want To Public License/i ) {
-			$license = "WTFPL $license";
-			push @spdx_license, "WTFPL";
+			$gen_license->('WTFPL');
 		}
 	}
 	given ($licensetext) {
 		when ( /(License WTFPL|Under (the|a) WTFPL)/i ) {
-			$license = "WTFPL $license";
-			push @spdx_license, "WTFPL";
+			$gen_license->('WTFPL');
 		}
 	}
 	#>>>
-	$license = "UNKNOWN" if ( !length($license) );
-	push @spdx_license, "UNKNOWN" unless @spdx_license;
 
 	# Remove trailing spaces.
 	$license =~ s/\s+$//;
-	return $self->deb_fmt ? join( ' or ', @spdx_license ) : $license;
+	return $self->deb_fmt ? join( ' and/or ', @spdx_license ) : $license;
 }
 
 =encoding UTF-8
